//
//  GLGeometry.h
//  GLGeometry
//
//  Created by Jeffrey Early on 4/15/06.
//  Copyright 2006, Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryTypes.h>
#import "geos_c.h"

/* GLGeometry */

// Entirely based on the "OpenGIS Simple Features Specification for SQL"

// Geometry is the root class of the hierarchy. Geometry is an abstract
// (non-instantiable) class.

// A private function must be written to convert from WKB format to the struct
// definition given in the standard. See ogrpoint.cpp for example. And the
// utility http://www.opengeospatial.org/resources/?page=testing&view=sfsql
@class GLMapProjection;
@interface GLGeometry : NSObject {

	GEOSGeom		geosGeometry;
	
	NSMutableDictionary*			scaledCocoaObjects;
	NSMutableDictionary*			transformedGeometries;
	NSMapTable*						projectedGeometries;
	NSMapTable*						clippedGeometries;
	
	NSData *wkbData;
	NSSet *leafNodes;
}

@property(copy) NSData *wkbData;

/************************************************/
/*		Object Initialization					*/
/************************************************/

#pragma mark -
#pragma mark Object Initialization
#pragma mark

// decodes the WKB information in the NSData and returns an appropriate object
// use "wkb2wkt.c" for implementation details
+ (id) geometryFromData: (NSData *) data;

// Creates and returns a GLGeometry object that contains a given GLPoint structure.
+ (GLGeometry *) geometryWithPoint: (GLPoint) aPoint;

// Creates and returns a GLGeometry object that contains a given GLMultiPoint structure.
+ (GLGeometry *) geometryWithMultiPoint: (GLMultiPoint) aMultiPoint;

// Creates and returns a GLGeometry object that contains a given GLLineString structure.
+ (GLGeometry *) geometryWithLineString: (GLLineString) aLineString;

// Creates and returns a GLGeometry object that contains a given GLPolygon structure.
+ (GLGeometry *) geometryWithPolygon: (GLPolygon) aPolygon;

// Creates and returns a GLGeometry object that contains a given GLMultiPolygon structure.
+ (GLGeometry *) geometryWithMultiPolygon: (GLMultiPolygon) aMultiPolygon;


// Creates and returns a GLGeometry object that contains the given GLPointPtr structure.
+ (GLGeometry *) geometryWithPointPtr: (GLPointPtr) aPoint;

// Creates and returns a GLGeometry object that contains the given GLMultiPointPtr structure.
+ (GLGeometry *) geometryWithMultiPointPtr: (GLMultiPointPtr) aMultiPoint;

// Creates and returns a GLGeometry object that contains the given GLPolygonPtr structure.
+ (GLGeometry *) geometryWithPolygonPtr: (GLPolygonPtr) aPolygon;

/************************************************/
/*		Special Accessor Methods				*/
/************************************************/

#pragma mark -
#pragma mark Special Accessor Methods
#pragma mark

- (GEOSGeom) geosGeometry;

// returns a set of GLTreeLeafNodes. Rather than taking the minimum bounding rect for the
// entire geometry, objects may break it into components, e.g., an MBR for each polygon in
// a multipolygon geometry.
@property(assign) NSSet * leafNodes;

// maps to various projected versions of the geometry
// for now, the key is just the mapView.
@property(readonly) NSMapTable * projectedGeometries;

// Checks this geometries minimum bounding rectangle against the bounds. Return YES if the
// rectangles overlap.
- (BOOL) mbrInsertsectsRect: (GLRect) bounds;

// Will return a new copy of the geometry in projected coordinates.
- (GLGeometry *) geometryWithProjection: (GLMapProjection *) mapProjection;

/************************************************/
/*		OpenGIS: Basic Methods					*/
/************************************************/

#pragma mark -
#pragma mark OpenGIS: Basic Methods
#pragma mark

// The inherent dimension of this Geometry object, which must be less than or
// equal to the coordinate dimension. This specification is restricted to 
// geometries in two-dimensional coordinate space.
- (int) dimension;

// Returns the name of the instantiable subtype of Geometry of which this 
// Geometry instance is a member.  The name of the instantiable subtype of
// Geometry is returned as a string.
- (GLGeometryType) geometryType;

// Returns the Spatial Reference System ID for this Geometry.
- (int) srid;

// The minimum bounding box for this Geometry, returned as a Geometry.  The 
// polygon is defined by the corner points of the bounding box ((MINX, MINY),
// (MAXX, MINY), (MAXX, MAXY), (MINX, MAXY), (MINX, MINY)).
- (GLGeometry *) envelope;
 
// Exports this Geometry to a specific well-known text representation of Geometry.
- (NSString *) asText;

// Exports this Geometry to a specific well-known binary representation of Geometry.
- (NSData *) asBinary;

// Returns 1 (TRUE) if this Geometry is the empty geometry . If true, then this 
// Geometry represents the empty point set for the coordinate space.
- (BOOL) isEmpty;

// Returns 1 (TRUE) if this Geometry has no anomalous geometric points, such as
// self intersection or self tangency. The description of each instantiable 
// geometric class will include the specific conditions that cause an instance 
// of that class to be classified as not simple.
// http://geometryalgorithms.com/Archive/algorithm_0108/algorithm_0108.htm
- (BOOL) isSimple;

// Returns the closure of the combinatorial boundary of this Geometry. The 
// combinatorial boundary is defined as described in section 3.12.3.2 of [1].
// Because the result of this function is a closure, and hence topologically 
// closed, the resulting boundary can be represented using representational 
// geometry primitives as discussed in [1], section 3.12.2.
- (GLGeometry *) boundary;

/************************************************/
/*		OpenGIS: Spatial Relations Methods		*/
/************************************************/

#pragma mark -
#pragma mark OpenGIS: Spatial Relations Methods
#pragma mark

- (BOOL) equals: (GLGeometry *) anotherGeometry;

- (BOOL) disjoin: (GLGeometry *) anotherGeometry;

- (BOOL) intersects: (GLGeometry *) anotherGeometry;

- (BOOL) touches: (GLGeometry *) anotherGeometry;

- (BOOL) crosses: (GLGeometry *) anotherGeometry;

- (BOOL) within: (GLGeometry *) anotherGeometry;

- (BOOL) contains: (GLGeometry *) anotherGeometry;

- (BOOL) overlaps: (GLGeometry *) anotherGeometry;

/************************************************/
/*		OpenGIS: Spatial Analysis Methods		*/
/************************************************/

#pragma mark -
#pragma mark OpenGIS: Spatial Analysis Methods
#pragma mark

// Returns the shortest distance between any two points in 
// the two geometries as calculated in the spatial reference system of this Geometry.
- (double) distance: (GLGeometry *) anotherGeometry;

// Returns a geometry that represents all points whose distance from 
// this Geometry is less than or equal to distance. Calculations are in the Spatial
// Reference System of this Geometry.
- (GLGeometry *) buffer: (double) distance;

// Returns a geometry that represents the convex hull of this Geometry.
- (GLGeometry *) convexHull;

// Returns a geometry that represents the point set intersection of this Geometry
// with anotherGeometry.
- (GLGeometry *) intersection: (GLGeometry *) anotherGeometry;

// Returns a geometry that represents the point set union of this Geometry with 
// anotherGeometry.
- (GLGeometry *) unionGeom: (GLGeometry *) anotherGeometry;

// Returns a geometry that represents the point set difference of this Geometry with 
// anotherGeometry.
- (GLGeometry *) difference: (GLGeometry *) anotherGeometry;

// Returns a geometry that represents the point set symmetric difference of this Geometry
// with anotherGeometry.
- (GLGeometry *) symmetricDifference: (GLGeometry *) anotherGeometry;

@end
