//
//  GLMapProjection.h
//  GLGeometry
//
//  Created by Jeffrey Early on 2/20/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryTypes.h>

/************************************************/
/*		GLMapProjection							*/
/************************************************/

#pragma mark -
#pragma mark GLMapProjection
#pragma mark

@interface GLMapProjection : NSObject <NSCopying> {
	NSInteger srid;
	NSString *name;
	BOOL equitorialCylindrical;
	BOOL UTM;
	GLRect bounds;
	BOOL wrapsHorizontally;
	BOOL wrapsVertically;
	double maximumResolution;
	double minimumResolution;
}

+ (GLMapProjection *) mapProjectionWithPlateCarree;
+ (GLMapProjection *) mapProjectionWithUTMZone: (NSUInteger) zone;

// The Spatial Reference Identification (srid) is a unique number identifying this projection type.
// One could certainly use the EPSG numbering scheme for consistency.
@property NSInteger srid;

// The name of the projection, eg, Transverse Mercator, Plate Carre
@property(copy) NSString *name;

// I believe it is correct to say that an equitorial cylindrical projection keeps North-South along
// the y axis and East-West along the x axis. Thus, one could draw longitude and
// latitude lines as straight lines.
@property (assign,getter=isEquitorialCylindrical) BOOL equitorialCylindrical;

// Whether or not this projection is one of the projections in the UTM system.
@property (assign,getter=isUTM) BOOL UTM;

// Bounds, in projected units, for which the projection is valid and whether it wraps,
// meaning, you can get to the east, by going west (eg, Mercator),
// or north, by going south (eg, Transverse Mercator), etc.
@property(assign) GLRect bounds;
@property(assign) BOOL wrapsHorizontally, wrapsVertically;

// The heart of the projection --
// Convert between projected coordinates (x,y) and WGS84 (longitude, latitude)
- (GLPoint) projectedCoordinateForLongitudeLatitude: (GLPoint) latLong;
- (GLPoint) longitudeLatitudeForProjectedCoordinate: (GLPoint) coordinate;

// Although this could be figured out from the above two methods, here the projection
// should return the number of meters in every projected unit at a given (lon, lat).
- (double) metersPerProjectedUnitForLongitudeLatitude: (GLPoint) latLong;

// Although technically not part of the concept for a projection, these allow one to
// place restrictions on reasonable usage. The results should be returned in projectedUnits/pixel.
// The geometries may use these to cache at various resolutions. Also, don't forget a maximum resolution
// is actually a lower number than a minimum resolution.
@property(assign) double maximumResolution;
@property(assign) double minimumResolution;

// For this is simply a check on the SRIDs of the respective projections.
- (BOOL) isEqualToMapProjection: (GLMapProjection *) anotherProjection;

@end