//
//  GLTextSymbolizer.h
//  GLMapKit
//
//  Created by Jeffrey Early on 12/31/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLSymbolizer.h>
#import <GLGeometryKit/GLGeometryTypes.h>

// The label should take the form @"${aVariable} city" where the keyPath "aVariable" will be used
// as a substitution variable.

typedef enum {
	kGLAnchorLeftType=0,
    kGLAnchorTopLeftType=1,
    kGLAnchorTopType=2,
    kGLAnchorTopRightType=3,
	kGLAnchorRightType=4,
	kGLAnchorBottomRightType=5,
	kGLAnchorBottomType=6,
	kGLAnchorBottomLeftType=7,
	kGLAnchorCenterType=8

} GLTextSymbolizerAnchorPositionType;

typedef enum {
	kGLLineCurvedOrientation=0,
	kGLLineParallelOrientation=1
	
} GLTextSymbolizerLineOrientationType;

typedef enum {
	kGLAboveLinePosition=0,
	kGLOnLinePosition=1,
	kGLBelowLinePosition=2,
	kGLAutoLinePosition=3
	
} GLTextSymbolizerLinePerpendicularPositionType;

typedef enum {
	kGLLeftLinePosition=0,
	kGLCenterLinePosition=1,
	kGLRightLinePosition=2
	
} GLTextSymbolizerLineParallelPositionType;

typedef enum {
	kGLPolygonHorizontalOrientation=0,
	kGLPolygonStraightOrientation=1,
	kGLPolygonHorizontalThenStraightOrientation=1
	
} GLTextSymbolizerPolygonOrientationType;

// These structures essentially emulate CoreText glyph positioning,
// but customized for our purposes. In particular each *label* may be drawn
// multiple times, hence the GLMultiTextLine structure. Each GLTextLine (each
// label) has an initial start point in projected coordinates. Each line has
// multiple runs (see CoreText), each potentially with a different font. We
// then have individual text positions.

typedef CGAffineTransform * __strong CGAffineTransformPtr;
typedef CGGlyph * __strong CGGlyphPtr;

typedef struct GLTextRun {
	CGFontRef font;
	CGFloat fontSize;
	
	NSUInteger numGlyphs;
	CGGlyphPtr glyphs;
	CGAffineTransformPtr textMatrices;
} GLTextRun;

typedef GLTextRun * __strong GLTextRunPtr;

typedef struct GLTextLine {
	GLPoint		projectedInitialPoint;
	CGSize		pixelSize; // before rotation
	CGAffineTransform	contextTransform; // to be applied after the above transform.
	
	NSUInteger	numTextRuns;
	GLTextRunPtr textRuns;
} GLTextLine;

typedef GLTextLine * __strong GLTextLinePtr;

typedef struct GLMultiTextLine {
	NSUInteger	numTextLines;
	GLTextLinePtr textLines;
} GLMultiTextLine;

typedef GLMultiTextLine * __strong GLMultiTextLinePtr;


@class GLSpatialObject;
@interface GLTextSymbolizer : GLSymbolizer <NSCoding> {
	
	NSFont *font;
	
	NSString *label;
	
	BOOL shouldFill;
	NSColor *fillColor;
	
	BOOL shouldStroke;
	NSColor *strokeColor;
	float strokeWidth;
	
	BOOL shouldShadow;
	NSShadow *shadow;
	
	BOOL shouldHalo;
	NSColor *haloColor;

	GLPoint offset;
	GLTextSymbolizerAnchorPositionType anchorPosition;
	float rotation;
	
	GLTextSymbolizerLineOrientationType lineOrientation;
	GLTextSymbolizerLinePerpendicularPositionType linePerpendicularPosition;
	GLTextSymbolizerLineParallelPositionType lineParallelPosition;
	float linePerpendicularOffset;
	BOOL shouldRepeat;
	float repeatSpacing;
	BOOL shouldDrawOnlyWhenFits;
	GLTextSymbolizerPolygonOrientationType polygonOrientation;
	
	GLMultiTextLinePtr multiTextLine;
	
	BOOL rotationAnchorOrOffsetIsDirty;
}

// Font properties for the text label
@property(copy) NSFont *font;
@property(copy) NSString *fontName;
@property float fontSize;

// The *raw* label, including wildcards, etc.
@property(copy) NSString *label;

// Color properties of the font
@property BOOL shouldFill, shouldStroke;
@property(copy) NSColor *fillColor, *strokeColor;
@property float fillOpacity, strokeOpacity;
@property float strokeWidth;

// Shadow properties of the font
@property BOOL shouldShadow;
@property(copy) NSShadow *shadow;
@property float shadowOffsetWidth, shadowOffsetHeight, shadowBlurRadius;
@property float shadowOffsetDistance, shadowOffsetRotation; // alternative to width and height.
@property(copy) NSColor *shadowColor;

// Halo properties, for now, I decide the details beyond this.
@property BOOL shouldHalo;
@property(copy) NSColor *haloColor;

// Positioning and orientation of the label
@property GLPoint offset;
@property float xOffset, yOffset;
@property GLTextSymbolizerAnchorPositionType anchorPosition;

// New additions (needs NSCoding, copying, etc.)
@property float rotation;

@property GLTextSymbolizerLineOrientationType lineOrientation;
@property GLTextSymbolizerLinePerpendicularPositionType linePerpendicularPosition;
@property GLTextSymbolizerLineParallelPositionType lineParallelPosition;
@property float linePerpendicularOffset;
@property BOOL shouldRepeat;
@property float repeatSpacing;

@property BOOL shouldDrawOnlyWhenFits;
@property GLTextSymbolizerPolygonOrientationType polygonOrientation;

- (NSString *) labelFromSpatialObject: (GLSpatialObject *) aSpatialObject;
- (NSAttributedString *) attributedLabelFromSpatialObject: (GLSpatialObject *) aSpatialObject;
- (BOOL) isEqualToTextSymbolizer: (GLTextSymbolizer *) aTextSymbolizer;

- (void) symbolizeInContext: (CGContextRef) context withScaleFactor: (double) scale lineAtIndex: (NSUInteger) index;
- (void) symbolizeHaloInContext: (CGContextRef) context withScaleFactor: (double) scale lineAtIndex: (NSUInteger) index;
GLTextLinePtr GLLineCreateWithAttributedString( CFAttributedStringRef string, CGContextRef context );
- (CGAffineTransform) transformWithPoint: (GLPoint) point forSize: (CGSize) labelSize withAnchor: (GLTextSymbolizerAnchorPositionType) position rotation: (float) aRotation;

@end
